package de.towerdefence.server.player.session;

import de.towerdefence.server.player.Player;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

@Component
public class PlayerLoginSessions {
    public static final int PLAYER_LOGIN_SESSION_TOKEN_BYTE_LENGTH = 64;
    private final SecureRandom random;
    private final Map<String, String> playerLoginSessionTokens = new HashMap<>();
    private final Map<String, Player> playerLoginSessionPlayers = new HashMap<>();
    private final Map<String, ScheduledFuture<?>> playerLoginSessionSchedule = new HashMap<>();
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);

    public PlayerLoginSessions() {
        random = new SecureRandom();
    }

    public String createPlayerLoginSession(Player player) {
        byte[] token_data = new byte[PLAYER_LOGIN_SESSION_TOKEN_BYTE_LENGTH];
        this.random.nextBytes(token_data);
        String token = new String(token_data, StandardCharsets.UTF_8);
        String playerName = player.getUsername();
        this.playerLoginSessionTokens.put(playerName, token);
        this.playerLoginSessionPlayers.put(playerName, player);
        this.playerLoginSessionSchedule.put(playerName, scheduler.schedule(
                () -> {
                    this.playerLoginSessionTokens.remove(playerName);
                    this.playerLoginSessionPlayers.remove(playerName);
                    this.playerLoginSessionSchedule.remove(playerName);
                },
                30,
                TimeUnit.SECONDS
        ));
        return token;
    }

    /**
     * @return an Optional Player. If it is empty, that Player has no valid Login Session
     */
    public Optional<Player> getPlayerFromLoginSession(String username, String token) {
        if (!this.playerLoginSessionTokens.containsKey(username)) {
            return Optional.empty();
        }
        if (!this.playerLoginSessionTokens.get(username).equals(token)) {
            return Optional.empty();
        }
        this.playerLoginSessionTokens.remove(username);
        Player player = this.playerLoginSessionPlayers.get(username);
        ScheduledFuture<?> task = this.playerLoginSessionSchedule.get(username);
        if (task != null) {
            task.cancel(true);
        }
        this.playerLoginSessionSchedule.remove(username);
        return Optional.of(player);
    }
}
